@@ -65,6 +65,13 @@ public:
     QTimer savingDelay;
 
 
+    enum DeletionState {
+        Normal,
+        DeletionBlocked,
+        DeletionScheduled
+    };
+    DeletionState deletionState = Normal;
+
 
     struct Data {
         QString name;
@@ -93,6 +100,50 @@ public:
     }
 
 
+    void writeConfiguration()
+    {
+        if (data) {
+
+            const auto deviceStr = device.data();
+            const auto mountPointStr = data->mountPoint.data();
+
+            Q_ASSERT(!deviceStr.isEmpty() && !mountPointStr.isEmpty());
+
+            // Saving the data for the current mount
+            KConfigGroup generalConfig(config, "EncryptedDevices");
+            generalConfig.writeEntry(deviceStr, true);
+
+            KConfigGroup vaultConfig(config, deviceStr);
+            vaultConfig.writeEntry(CFG_LAST_STATUS, (int)data->status);
+            vaultConfig.writeEntry(CFG_MOUNT_POINT, mountPointStr);
+            vaultConfig.writeEntry(CFG_NAME,        data->name);
+            vaultConfig.writeEntry(CFG_BACKEND,     data->backend->name());
+
+            vaultConfig.writeEntry(CFG_ACTIVITIES,  data->activities);
+            vaultConfig.writeEntry(CFG_OFFLINEONLY, data->isOfflineOnly);
+
+        } else {
+
+            KConfigGroup generalConfig(config, "EncryptedDevices");
+            generalConfig.writeEntry(device.data(), false);
+
+            KConfigGroup vaultConfig(config, device.data());
+            vaultConfig.writeEntry(CFG_LAST_STATUS, (int)VaultInfo::Error);
+            vaultConfig.writeEntry(CFG_LAST_ERROR,
+                    data.error().message() + " (code: " +
+                    QString::number(data.error().code()) + ")");
+            // vaultConfig.deleteEntry(CFG_MOUNT_POINT);
+            // vaultConfig.deleteEntry(CFG_NAME);
+            // vaultConfig.deleteEntry(CFG_BACKEND);
+            // vaultConfig.deleteEntry(CFG_ACTIVITIES);
+            // vaultConfig.deleteEntry(CFG_OFFLINEONLY);
+
+        }
+
+        config->sync();
+    }
+
+
     void updateStatus()
     {
         if (data) {
@@ -121,11 +172,13 @@ public:
 
                 emit q->statusChanged(data->status);
 
-                if (newStatus == VaultInfo::Closed || newStatus == VaultInfo::Opened) {
+                if (newStatus == VaultInfo::Closed
+                        || newStatus == VaultInfo::Opened) {
                     emit q->isOpenedChanged(newStatus == VaultInfo::Opened);
                 }
 
-                if (oldStatus == VaultInfo::NotInitialized || newStatus == VaultInfo::NotInitialized) {
+                if (oldStatus == VaultInfo::NotInitialized
+                        || newStatus == VaultInfo::NotInitialized) {
                     emit q->isInitializedChanged(newStatus);
                 }
 
@@ -136,18 +189,7 @@ public:
                     emit q->isBusyChanged(false);
                 }
 
-                // Saving the data for the current mount
-                KConfigGroup generalConfig(config, "EncryptedDevices");
-                generalConfig.writeEntry(device.data(), true);
-
-                KConfigGroup vaultConfig(config, device.data());
-                vaultConfig.writeEntry(CFG_LAST_STATUS, (int)data->status);
-                vaultConfig.writeEntry(CFG_MOUNT_POINT, data->mountPoint.data());
-                vaultConfig.writeEntry(CFG_NAME,        data->name);
-                vaultConfig.writeEntry(CFG_BACKEND,     data->backend->name());
-
-                vaultConfig.writeEntry(CFG_ACTIVITIES,  data->activities);
-                vaultConfig.writeEntry(CFG_OFFLINEONLY, data->isOfflineOnly);
+                writeConfiguration();
 
                 org::kde::KDirNotify::emitFilesAdded(
                         QUrl::fromLocalFile(data->mountPoint.data()));
@@ -158,24 +200,10 @@ public:
             emit q->isInitializedChanged(false);
             emit q->isBusyChanged(false);
 
-            KConfigGroup generalConfig(config, "EncryptedDevices");
-            generalConfig.writeEntry(device.data(), false);
-
-            KConfigGroup vaultConfig(config, device.data());
-            vaultConfig.writeEntry(CFG_LAST_STATUS, (int)VaultInfo::Error);
-            vaultConfig.writeEntry(CFG_LAST_ERROR,
-                    data.error().message() + " (code: " +
-                    QString::number(data.error().code()) + ")");
-            // vaultConfig.deleteEntry(CFG_MOUNT_POINT);
-            // vaultConfig.deleteEntry(CFG_NAME);
-            // vaultConfig.deleteEntry(CFG_BACKEND);
-            // vaultConfig.deleteEntry(CFG_ACTIVITIES);
-            // vaultConfig.deleteEntry(CFG_OFFLINEONLY);
+            writeConfiguration();
 
             emit q->statusChanged(VaultInfo::Error);
         }
-
-        config->sync();
     }
 
 
@@ -207,13 +235,17 @@ public:
         vaultData.status = VaultInfo::Error;
 
         // Reading the mount data from the config
-        const KConfigGroup vaultConfig(config, device.data());
+        KConfigGroup vaultConfig(config, device.data());
         vaultData.name          = vaultConfig.readEntry(CFG_NAME, name);
-        vaultData.mountPoint    = MountPoint(vaultConfig.readEntry(CFG_MOUNT_POINT, mountPoint.data()));
         vaultData.backendName   = vaultConfig.readEntry(CFG_BACKEND, backendName);
         vaultData.activities    = vaultConfig.readEntry(CFG_ACTIVITIES, activities);
         vaultData.isOfflineOnly = vaultConfig.readEntry(CFG_OFFLINEONLY, isOfflineOnly);
 
+        const QString configuredMountPoint = vaultConfig.readEntry(CFG_MOUNT_POINT, mountPoint.data());
+        vaultData.mountPoint = MountPoint(configuredMountPoint);
+        const QString actualMountPoint = vaultData.mountPoint.data();
+        vaultConfig.writeEntry(CFG_MOUNT_POINT, actualMountPoint);
+
         const QDir mountPointDir(vaultData.mountPoint);
 
 
@@ -263,8 +295,15 @@ public:
         emit q->isBusyChanged(true);
         data->status = whileNotFinished;
 
+        deletionState = DeletionBlocked;
+
         return future | onSuccess([this] {
                 updateStatus();
+
+                if (deletionState == DeletionScheduled) {
+                    q->deleteLater();
+                }
+                deletionState = Normal;
             });
     }
 
@@ -293,20 +332,7 @@ Vault::Vault(const Device &device, QObject *parent)
     d->savingDelay.setSingleShot(true);
     connect(&d->savingDelay, &QTimer::timeout,
             this, [&] {
-                qDebug() << "Saving vault info:"
-                         << d->data->name
-                         << d->data->mountPoint
-                         << d->data->activities
-                         << d->data->isOfflineOnly
-                         ;
-                KConfigGroup vaultConfig(d->config, d->device.data());
-                vaultConfig.writeEntry(CFG_MOUNT_POINT, d->data->mountPoint.data());
-                vaultConfig.writeEntry(CFG_NAME,        d->data->name);
-                vaultConfig.writeEntry(CFG_ACTIVITIES,  d->data->activities);
-                vaultConfig.writeEntry(CFG_OFFLINEONLY, d->data->isOfflineOnly);
-
-                d->config->sync();
-
+                d->writeConfiguration();
                 emit infoChanged();
             });
 }
@@ -322,6 +348,17 @@ Vault::~Vault()
 
 
 
+void Vault::scheduleDeletion()
+{
+    if (d->deletionState == Private::Normal) {
+        deleteLater();
+    } else {
+        d->deletionState = Private::DeletionScheduled;
+    }
+}
+
+
+
 void Vault::saveConfiguration()
 {
     d->savingDelay.start();
@@ -362,6 +399,42 @@ FutureResult<> Vault::create(const QString &name, const MountPoint &mountPoint,
 }
 
 
+
+FutureResult<> Vault::import(const QString &name, const MountPoint &mountPoint,
+                             const Payload &payload)
+{
+    using namespace AsynQt::operators;
+
+    return
+        // If the backend is already known, and the device is initialized,
+        // we do not want to do it again
+        d->data && (!d->data->backend->isInitialized(d->device)) ?
+            errorResult(Error::DeviceError,
+                        i18n("This device is not initialized. Cannot import it.")) :
+
+        // Mount not open, check the error messages
+        !(d->data = d->loadVault(d->device, name, mountPoint, payload)) ?
+            errorResult(Error::BackendError,
+                        i18n("Unknown error, unable to create the backend.")) :
+
+        // otherwise
+        d->followFuture(VaultInfo::Creating,
+                        d->data->backend->import(name, d->device, mountPoint, payload))
+            | onSuccess([mountPoint] {
+                // If we have successfully created the vault,
+                // lets try to set its icon
+                QFile dotDir(mountPoint + "/.directory");
+
+                if (dotDir.open(QIODevice::WriteOnly | QIODevice::Text)) {
+                    QTextStream out(&dotDir);
+                    out << "[Desktop Entry]\nIcon=folder-decrypted\n";
+                }
+            });
+}
+
+
+
+
 FutureResult<> Vault::open(const Payload &payload)
 {
     return
@@ -460,7 +533,7 @@ FutureResult<> Vault::forceClose()
             | onError([this] {
                 d->updateMessage(i18n("Failed to fetch the list of applications using this vault"));
             })
-            | onSuccess([this] (const QString &result) {
+            | onSuccess([] (const QString &result) {
                 // based on ksolidnotify.cpp
                 QStringList blockApps;
 
