@@ -37,9 +37,8 @@ public:
     ~PlasmaVaultService();
 
 public Q_SLOTS:
-    Q_SCRIPTABLE void init();
-
     Q_SCRIPTABLE void requestNewVault();
+    Q_SCRIPTABLE void requestImportVault();
     Q_SCRIPTABLE void openVault(const QString &device);
     Q_SCRIPTABLE void closeVault(const QString &device);
     Q_SCRIPTABLE void forceCloseVault(const QString &device);
