@@ -35,8 +35,8 @@ class VaultsModel: public QAbstractListModel {
     Q_PROPERTY(bool hasError READ hasError NOTIFY hasErrorChanged)
 
 public:
-    VaultsModel(QObject *parent = nullptr);
-    ~VaultsModel();
+    explicit VaultsModel(QObject *parent = nullptr);
+    ~VaultsModel() override;
 
     int rowCount(const QModelIndex &parent) const override;
 
@@ -53,6 +53,7 @@ public:
         VaultIsOpened,                ///< Is the vault open
         VaultStatus,                  ///< Status of the vault, @see VaultInfo::Status
         VaultActivities,              ///< To which activities does this vault belong to
+        VaultIsOfflineOnly,           ///< Should the networking be off when this vault is mounted
         VaultMessage                  ///< Message to be shown forthe vault
     };
 
@@ -100,7 +101,7 @@ class SortedVaultsModelProxy: public QSortFilterProxyModel {
     Q_OBJECT
 
 public:
-    SortedVaultsModelProxy(QObject *parent);
+    explicit SortedVaultsModelProxy(QObject *parent);
 
     bool lessThan(const QModelIndex &left,
                   const QModelIndex &right) const override;
